# Using MML open WMTS with react-map-gl (mapbox-gl)

NOTE! At the moment the project uses the
[raster tile WMTS](https://www.maanmittauslaitos.fi/kartat-ja-paikkatieto/asiantuntevalle-kayttajalle/kartta-ja-paikkatietojen-rajapintapalvelut-19),
but it seems that MML has [vector tile WMTS](https://beta-karttakuva.maanmittauslaitos.fi/demo/)
in beta.


UPDATE! Just for testing, I updated the code to use the vector tile beta api.
The mapbox configuration is copied from here
https://beta-karttakuva.maanmittauslaitos.fi/vtbasemaps/data/NLSbasemapWithRaster.json


This project uses [react-map-gl](https://github.com/uber/react-map-gl) library
for rendering the basemap for the application. The `react-map-gl` library is
just a light react wrapper on top of the [mapbox-gl-js](https://github.com/mapbox/mapbox-gl-js)
library. The `mapbox-gl-js` library is created to render maps from
[mapbox](https://www.mapbox.com/), but the project is open source and is easily
configurable to use maps from other sources.


To use the raster tiles from MML WMTS, just configure new source to the
`mapbox-gl-js` [style configuration file](../../public/mml-basemap.json). The
url in the `tiles` property is the important part. It tells the library where
to fetch map tile for any arbitrary coordinates. The MML WMTS actually has two
different paths for two different coordinate systems (`EPSG:3067` and `WGS84`).
It's important to use the `WGS84` path (`WGS84_Pseudo-Mercator` in the url),
for the map view to work correctly. The style json is given as the `mapStyle`
property for the `StaticMap` component. Working example from the app:

```
<DeckGL
  {...this.state.viewport}
  {...this.props.mapSize}
  layers={layers}
  controller={MapController}
  onViewportChange={v => this.setState({viewport: v})}
>
  <StaticMap
    {...this.props.mapSize}
    mapStyle="mml-basemap.json"
    viewState={this.state.viewport}
  />
</DeckGL>
```

Similar to this example, the `mapbox-gl-js` should work with any WMTS as long
as the url is correct and the coordinate system is `WGS84`. Actually, the
`StaticMap` component has `transformRequest` property that maybe can be used to
convert the coordinates? I have not tested this though.

# Self hosting map tiles

As can be seen from the example, WMTS is a very simple API. It does not need
any active processing at the server end. The map tiles can be prerendered and
served with any web server as static content. This makes self serving the tiles
a trivial task. The hard part is to create the correct folder structure and get
the tile images. [tilelive](https://github.com/mapbox/tilelive) and it's
[tilelive-tms](https://github.com/oscarfonts/tilelive-tms) module seem to be
able to do this automatically. I did not test the tool but the WMTS example in
the repository should work with MML WMTS too.
