import Rainbow from 'rainbowvis.js';

const color_spectrum = ["orange", "yellow", "green"];
const undef_color = "grey";

class LamStation {

  constructor(id, coordinates, avg_speed_dir1, avg_speed_dir2, data) {
    this.id = id;
    this.coordinates = coordinates;
    this.avg_speed_dir1 = avg_speed_dir1;
    this.avg_speed_dir2 = avg_speed_dir2;
    this.free_flow_speed_dir1 = null;
    this.free_flow_speed_dir2 = null;
    this.traffic_vol_dir1 = null;
    this.traffic_vol_dir2 = null;
    this.rainbow = new Rainbow();
    this.rainbow.setSpectrum(...color_spectrum);
    this.rainbow.setNumberRange(0.0, 1.0);
    this.data = data
  }

  set_free_flow_speed_dir1(free_flow_speed) {
    this.free_flow_speed_dir2 = free_flow_speed;
  }

  set_free_flow_speed_dir2(free_flow_speed) {
    this.free_flow_speed_dir1 = free_flow_speed;
  }

  set_avg_speed_dir1(avg_speed) {
    this.avg_speed_dir2 = avg_speed;
  }

  set_avg_speed_dir2(avg_speed) {
    this.avg_speed_dir1 = avg_speed;
  }

  set_traffic_vol_dir1(volume) {
    this.traffic_vol_dir1 = volume;
  }

  set_traffic_vol_dir2(volume) {
    this.traffic_vol_dir2 = volume;
  }

  get_coordinates() {
    return this.coordinates;
  }

  get_id() {
    return this.id;
  }

  get_color() {
    const grey = [211, 211, 211];
    const free_flow1 = this.get_free_flow_speed_dir1();
    const free_flow2 = this.get_free_flow_speed_dir2();
    const avg_speed1 = this.get_avg_speed_dir1();
    const avg_speed2 = this.get_avg_speed_dir2();
    if (avg_speed1 === null || avg_speed2 === null) {
      return grey;
    } else if (free_flow1 === null || free_flow2 === null) {
      return grey;
    } else if (free_flow1 === 0 || free_flow2 === 0) {
      return grey;
    } else if (Math.min(avg_speed1, avg_speed2) === 0) {
      return grey;
    }
    const ratio1 = avg_speed1 / free_flow1;
    const ratio2 = avg_speed2 / free_flow2;
    const min_ratio = Math.min(ratio1, ratio2);
    let hex = this.rainbow.colorAt(min_ratio);
    let bigint = parseInt(hex, 16);
    let r = (bigint >> 16) & 255;
    let g = (bigint >> 8) & 255;
    let b = bigint & 255;
    return [r, g, b];
  }

  get_avg_speed() {
    const avg_speed1 = this.get_avg_speed_dir1();
    const avg_speed2 = this.get_avg_speed_dir2();
    if (avg_speed1 === null || avg_speed2 === null) {
      return 0;
    }
    return (this.avg_speed_dir1 + this.avg_speed_dir1) / 2;
  }

  get_avg_traffic_vol() {
    return (this.traffic_vol_dir1 + this.traffic_vol_dir2) / 2;
  }

  get_free_flow_speed_dir1() {
    return this.free_flow_speed_dir1;
  }

  get_free_flow_speed_dir2() {
    return this.free_flow_speed_dir2;
  }

  get_avg_speed_dir1() {
    return this.avg_speed_dir1;
  }

  get_avg_speed_dir2() {
    return this.avg_speed_dir2;
  }

  get_traffic_vol_dir1() {
    return this.traffic_vol_dir1;
  }

  get_traffic_vol_dir2() {
    return this.traffic_vol_dir2;
  }

  get_name() {
    return this.data.properties.name;
  }

}

export function get_color_spectrum() {
  return color_spectrum.slice();
}

export function get_undef_color() {
  return undef_color;
}

export default LamStation;
